
#ifndef _CST_H_
#define _CST_H_

#define WORK_DIR "/tmp/"
#define RESOURCE_DIR "/usr/local/share/cst/"
#define MOUNT_DIR "/mnt/"
#define STORAGE_DIR "/mnt/cst-data/"

#if defined(USE_SYSLOG)
#include <syslog.h>
#endif

#include "cst_survey.h"

enum {
	PROC_RESULT_ERROR	= 0x0000,
	PROC_RESULT_NEXT	= 0x0001,
	PROC_RESULT_CANCEL	= 0x0002,
	PROC_RESULT_ALLCANCEL	= 0x0004,
	PROC_RESULT_EXIT	= 0x0008,
};

enum {
	PROC_PAGE_START		= 0x0001,
	PROC_PAGE_FINISH	= 0xffff,
};

#if defined(USE_SYSLOG)
	#define CSTERR(args...)		syslog(LOG_ERR, args)
	#define CSTWARN(args...)	syslog(LOG_WARNING, args)
	#define CSTINFO(args...)	syslog(LOG_INFO, args)
	#if defined(DEBUG_ENABLE)
		#define CSTDEBUG(args...)	syslog(LOG_DEBUG, args)
	#else
		#define CSTDEBUG(args...)
	#endif
#else
	#define CSTERR(args...)		printf("<ERR>  CST: " args)
	#define CSTWARN(args...)	printf("<WARN> CST: " args)
	#define CSTINFO(args...)	printf("<INFO> CST: " args)
	#if defined(DEBUG_ENABLE)
		#define CSTDEBUG(args...)	printf("<DEBUG>CST: " args)
	#else
		#define CSTDEBUG(args...)
	#endif
#endif

#define CSTFUNC() CSTDEBUG("FUNC(%s)\n", __FUNCTION__)
#define CSTLINE() CSTDEBUG("CHECK(%d)\n", __LINE__)

typedef struct {
	int type;
	int page;
	void *data;
} CSTProcResult_t;

extern char *CSTGetIPaddrString(void);
extern char *CSTGetMACaddrString(void);

extern void CSTSetFileLabel(char *);
extern char * CSTGetFileLabel(void);
extern char * CSTRequestLabel(void);
extern void CSTReleaseLabel(char *);

extern void CSTStartWindow_ReleaseResource(void);
extern int CSTStartWindow_PrepareResource(void);
extern int CSTStartWindow_Create(int);
extern int CSTStartWindow_Proc(int, CSTProcResult_t *);

extern void CSTSurveyWindow_PrepareCard(char *);
extern void CSTSurveyWindow_ReleaseResource(void);
extern int CSTSurveyWindow_PrepareResource(void);
extern int CSTSurveyWindow_Create(int);
extern int CSTSurveyWindow_Proc(int, CSTProcResult_t *);

extern void CSTEndWindow_ReleaseResource(void);
extern int CSTEndWindow_PrepareResource(void);
extern int CSTEndWindow_Create(void);
extern int CSTEndWindow_Wait(int);

#endif /* _CST_H_ */
