#include <directfb.h>

#include <sys/types.h>
#include <sys/wait.h>

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <fcntl.h>

#include "cst.h"
#include "cst_dfb_common.h"
#include "cst_card.h"
#include "cst_storage.h"

static pid_t pid_child = -1;
static char *current_card_label;
static CSTCardResult_t cardres;

static int
CSTCard_Exec(void)
{
	int ret;
	pid_t pid;

	CSTFUNC();

	current_card_label = CSTRequestLabel();

	pid_child = pid = fork();
	if (pid == -1) {
		pid_child = -1;
		return -1;
	}

	if (pid == 0) {
		char *args[] = {"scanimage", "--resolution", "150", "-l", "4",
				"-x", "95", "-y", "62", NULL};

		char file[FILENAME_MAX];
		int fd;

		sprintf(file, "%s.pnm", current_card_label);
		CSTDEBUG("CMD(scanimage > %s)\n", file);
		CSTINFO("SCAN(%s)\n", file);

		close(1);
		close(2);
		fd = open(file, O_WRONLY | O_CREAT, 0644);
		dup2(fd, 1);

		ret = execvp(args[0], args);

		exit(1);
	}

	return 0;
}

/* signal handler for SIGCHLD */
static void
sighandler(int sig, siginfo_t *info, void *arg)
{
	char file[FILENAME_MAX];
	int status;
	int ret;

	DFBUserEvent uevt;

	if (info->si_pid == pid_child) {
		ret = waitpid(info->si_pid, &status, 0);
		if (ret == 0 || ret == -1)
			return;

		if (WIFEXITED(status) == 1) {
			if (WEXITSTATUS(status) != 0)
				return;
		} else
			return;

		/* Convert */
		{
			char cmd[FILENAME_MAX];
			sprintf(cmd, "convert %s.pnm %s.jpg",
				current_card_label, current_card_label);
			system(cmd);
			CSTDEBUG("CMD(%s)\n", cmd);
			CSTINFO("CONV(%s.jpg)\n", current_card_label);

			sprintf(cmd, "rm -f %s.pnm",
				current_card_label);
			system(cmd);
		}

		/* Save JPEG file */
		sprintf(file, "%s.jpg", current_card_label);
		ret = CSTStorage_SaveFile(file);

		cardres.label = current_card_label;
		cardres.code = ret;

		/*Event*/
		uevt.clazz = DFEC_USER;
		uevt.type = DUET_INSERTCARD;
		uevt.data = (void *)&cardres;
		CSTDFBCommon_EventPost(DFB_EVENT(&uevt));

		CSTCard_Exec();
	} else
		waitpid(info->si_pid, NULL, 0);
}

int
CSTCard_Start(void)
{
	struct sigaction action;

	CSTFUNC();

	memset(&action, 0, sizeof(action));

	action.sa_sigaction = sighandler;
	action.sa_flags = SA_SIGINFO;
	sigemptyset(&action.sa_mask);

	sigaction(SIGCHLD, &action, NULL);

	CSTCard_Exec();

	return 0;
}

void
CSTCard_Stop(void)
{
	if (pid_child != -1)
		kill(pid_child, SIGINT);
}
