#include <directfb.h>

#include <stdio.h>
#include <unistd.h>

#include "cst.h"
#include "cst_dfb_common.h"

float VRatio_width;
float VRatio_height;

static IDirectFB *dfb;
static IDirectFBDisplayLayer *layer;

static IDirectFBSurface *cursor;

ICSTFont cstfont_s;
ICSTFont cstfont_m;
ICSTFont cstfont_l;

static int CSTEventState = 0;
IDirectFBEventBuffer *CSTEvent;

int
CSTDFBCommon_DrawString(IDirectFBSurface *surface, ICSTFont *cst_font,
			int color, char *str, int x, int y)
{
	struct font_color {
		u8 r; u8 g; u8 b; u8 a;
	} fc[] = {
		[CSTFONT_BLACK]		= { 0x00, 0x00, 0x00, 0xff},
		[CSTFONT_RED]		= { 0xff, 0x00, 0x00, 0xff},
		[CSTFONT_GREEN]		= { 0x00, 0xff, 0x00, 0xff},
		[CSTFONT_YELLOW]	= { 0xff, 0xff, 0x00, 0xff},
		[CSTFONT_BLUE]		= { 0x00, 0x00, 0xff, 0xff},
		[CSTFONT_PURPLE]	= { 0xff, 0x00, 0xff, 0xff},
		[CSTFONT_AQUA]		= { 0x00, 0xff, 0xff, 0xff},
		[CSTFONT_WHITE]		= { 0xff, 0xff, 0xff, 0xff},
	};
	DFBCHECK(surface->SetFont(surface, cst_font->font));
  
	surface->SetColor(surface,
			  fc[color].r, fc[color].g, fc[color].b, fc[color].a);
	surface->DrawString(surface, str, -1, x, y,
			    DSTF_LEFT | DSTF_TOP);
	return 0;
}

int
CSTDFBCommon_WarpCursor(int x, int y)
{
	layer->WarpCursor(layer, x, y);
	return 0;
}

int
CSTDFBCommon_WindowActivate(HWND_t *hWnd)
{
	hWnd->win->SetOpacity(hWnd->win, 0xff);
	return 0;
}

int
CSTDFBCommon_WindowInactivate(HWND_t *hWnd)
{
	hWnd->win->SetOpacity(hWnd->win, 0x00);
	return 0;
}

int
CSTDFBCommon_UpdateSurface(HWND_t *hWnd, IDirectFBSurface *surface)
{
	hWnd->master_sf->Blit(hWnd->master_sf, surface, NULL, 0, 0);
	return 0;
}

int
CSTDFBCommon_LoadImage(IDirectFBSurface *surface, char *image_file)
{
	IDirectFBImageProvider *provider;

	DFBCHECK(dfb->CreateImageProvider(dfb, image_file, &provider));
	provider->RenderTo(provider, surface, NULL);  
	provider->Release(provider);

	return 0;
}

void
CSTDFBCommon_ReleaseWindow(HWND_t *hWnd)
{
	hWnd->master_sf->Release(hWnd->master_sf);
	hWnd->primary_sf->Release(hWnd->primary_sf);
	hWnd->secondary_sf->Release(hWnd->secondary_sf);
	hWnd->win->Release(hWnd->win);
}

int
CSTDFBCommon_CreateWindow(HWND_t *hWnd,
			  int x, int y, int width, int height, EventProc *proc)
{
	DFBWindowDescription win_desc;
	DFBSurfaceDescription sf_desc;

	win_desc.flags  = (DWDESC_POSX | DWDESC_POSY |
			   DWDESC_WIDTH | DWDESC_HEIGHT | DWDESC_CAPS);
	win_desc.posx   = x;
	win_desc.posy   = y;
	win_desc.width  = width;
	win_desc.height = height;
	win_desc.caps   = DWCAPS_ALPHACHANNEL;

	DFBCHECK(layer->CreateWindow(layer, &win_desc, &hWnd->win));
	hWnd->win->GetSurface(hWnd->win, &hWnd->master_sf);

	/* create surfaces */
	sf_desc.flags  = (DWDESC_WIDTH | DWDESC_HEIGHT);
	sf_desc.width  = width;
	sf_desc.height = height;

	DFBCHECK(dfb->CreateSurface(dfb, &sf_desc, &hWnd->primary_sf));
	DFBCHECK(dfb->CreateSurface(dfb, &sf_desc, &hWnd->secondary_sf));

	hWnd->win->SetOpacity(hWnd->win, 0x00);
	hWnd->win->GetID(hWnd->win, &hWnd->id);

	hWnd->proc = proc;

	return 0;
}

void
CSTDFBCommon_EventClear(void)
{
	DFBEvent evt;

	if (CSTEventState) {
		while (CSTEvent->HasEvent(CSTEvent) == DFB_OK)
			DFBCHECK(CSTEvent->GetEvent(CSTEvent, &evt));
	}
}

void
CSTDFBCommon_EventPost(DFBEvent *event)
{
	if (CSTEventState)
		CSTEvent->PostEvent(CSTEvent, event);
}

void
CSTDFBCommon_EventInitialize(void)
{
	CSTEventState = 0;
	CSTEvent->Release(CSTEvent);
}

int
CSTDFBCommon_EventRegistration(HWND_t *hWnd)
{
	if (hWnd->proc) {
		if (!CSTEventState) {
			hWnd->win->CreateEventBuffer(hWnd->win, &CSTEvent);
			CSTEventState = 1;
		} else
			hWnd->win->AttachEventBuffer(hWnd->win, CSTEvent);
	}
	return 0;
}


int
CSTDFBCommon_PopupMsg(HBG_t *hBack, int width, char *msg, int sec)
{
	HWND_t hPopupMsg;
	DFBRectangle rect;

	rect.x = VPOSX((640 - width) / 2);
	rect.y = VPOSY(200);
	rect.w = VPOSX(width);
	rect.h = VPOSY(35 + 2 + cstfont_m.FontHeight);

	CSTDFBCommon_CreateWindow(&hPopupMsg,
				  rect.x, rect.y, rect.w, rect.h,
				  NULL);
	hPopupMsg.master_sf->SetColor(hPopupMsg.master_sf,
				      0x88, 0x88, 0x88, 0xff);
	hPopupMsg.master_sf->FillRectangle(hPopupMsg.master_sf,
					   0, 0, rect.w, rect.h);
	hPopupMsg.master_sf->SetColor(hPopupMsg.master_sf,
				      0xff, 0xff, 0xff, 0xff);
	hPopupMsg.master_sf->FillRectangle(hPopupMsg.master_sf,
					   1, 20, rect.w - 2, rect.h - 20 - 2);

	DFBCHECK(hPopupMsg.master_sf->SetFont(hPopupMsg.master_sf,
					      cstfont_m.font));
	hPopupMsg.master_sf->SetColor(hPopupMsg.master_sf,
				      0x00, 0x00, 0x00, 0xff);
	hPopupMsg.master_sf->DrawString(hPopupMsg.master_sf, msg, -1, 5, 25,
					DSTF_TOP | DSTF_LEFT);

	CSTDFBCommon_WindowActivate(&hPopupMsg);
	CSTDFBCommon_UpdateBackground(hBack);

	sleep(sec);

	CSTDFBCommon_ReleaseWindow(&hPopupMsg);
	CSTDFBCommon_UpdateBackground(hBack);

	return 0;
}

int
CSTDFBCommon_UpdateBackground(HBG_t *hBack)
{
	layer->SetBackgroundImage(layer, hBack->master_sf);
	layer->SetBackgroundMode(layer, DLBM_IMAGE);
	return 0;
}

void
CSTDFBCommon_RefreshBackground(HBG_t *hBack)
{
	hBack->master_sf->Blit(hBack->master_sf,
			       hBack->primary_sf, NULL, 0, 0);
}

void
CSTDFBCommon_ReleaseBackground(HBG_t *hBack)
{
	hBack->master_sf->Release(hBack->master_sf);
	hBack->primary_sf->Release(hBack->primary_sf);
}

int
CSTDFBCommon_CreateBackground(HBG_t *hBack, char *image_file)
{
	DFBDisplayLayerConfig config;
	DFBSurfaceDescription desc;

	layer->GetConfiguration(layer, &config);

	desc.flags = DSDESC_WIDTH | DSDESC_HEIGHT;
	desc.width = config.width;
	desc.height = config.height;

	DFBCHECK(dfb->CreateSurface(dfb, &desc, &hBack->master_sf));
	DFBCHECK(dfb->CreateSurface(dfb, &desc, &hBack->primary_sf));

	CSTDFBCommon_LoadImage(hBack->primary_sf, image_file); 

	CSTDFBCommon_RefreshBackground(hBack);

	return 0;
}

static int
CSTDFBCommon_CreateFont(IDirectFB *dfb, ICSTFont *cst_font,
			char *font_file, int font_height)
{
	DFBFontDescription desc;

	desc.flags = DFDESC_HEIGHT;
	desc.height = font_height;
	DFBCHECK(dfb->CreateFont(dfb, font_file, &desc, &cst_font->font));
	cst_font->FontHeight = font_height;
	cst_font->NextLineOffset = font_height + 1;

	return 0;
}

static int
CSTDFBCommon_CreateCursor(IDirectFB *dfb, IDirectFBDisplayLayer *layer,
			  IDirectFBSurface **cursor, char *cursor_file)
{
	DFBSurfaceDescription desc;
	IDirectFBImageProvider *provider;

	DFBCHECK(dfb->CreateImageProvider( dfb, cursor_file, &provider));
	DFBCHECK(provider->GetSurfaceDescription(provider, &desc));
	DFBCHECK(dfb->CreateSurface(dfb, &desc, cursor));
	provider->RenderTo(provider, *cursor, NULL);
	provider->Release(provider);
	layer->SetCursorShape(layer, *cursor, 0, 0);

	return 0;
}

int
CSTDirectFBSetup(int argc, char *argv[])
{
	DFBGraphicsDeviceDescription desc;
	DFBDisplayLayerConfig config;

	DFBCHECK(DirectFBInit(&argc, &argv));
	DFBCHECK(DirectFBCreate(&dfb));

	dfb->GetDeviceDescription(dfb, &desc);

	DFBCHECK(dfb->GetDisplayLayer(dfb, DLID_PRIMARY, &layer));

	layer->SetCooperativeLevel( layer, DLSCL_ADMINISTRATIVE );

	if (!((desc.blitting_flags & DSBLIT_BLEND_ALPHACHANNEL) &&
	      (desc.blitting_flags & DSBLIT_BLEND_COLORALPHA))) {
		config.flags = DLCONF_BUFFERMODE;
		config.buffermode = DLBM_BACKSYSTEM;

		layer->SetConfiguration(layer, &config);
	}
	layer->GetConfiguration(layer, &config);

	VRatio_width = (config.width / VWIDTH);
	VRatio_height = (config.height / VHEIGHT);

	CSTINFO("Layer.width: %d (%.2f)\n", config.width, VRatio_width);
	CSTINFO("Layer.height: %d (%.2f)\n", config.height, VRatio_height);

	CSTDFBCommon_CreateCursor(dfb, layer, &cursor,
				  "/usr/local/share/cst/cursor_arrow.png");

	layer->EnableCursor(layer, 1);
  
	/* Font Setup */
	CSTDFBCommon_CreateFont(dfb, &cstfont_s,
				RESOURCE_DIR"sazanami-gothic.ttf",
				config.height/(30 + 1));
	CSTDFBCommon_CreateFont(dfb, &cstfont_m,
				RESOURCE_DIR"sazanami-gothic.ttf",
				config.height/(20 + 1));
	CSTDFBCommon_CreateFont(dfb, &cstfont_l,
				RESOURCE_DIR"sazanami-gothic.ttf",
				config.height/(15 + 1));

	return 0;
}

int
CSTDirectFBRelease(void)
{
	CSTFUNC();
	cstfont_s.font->Release(cstfont_s.font);
	cstfont_m.font->Release(cstfont_m.font);
	cstfont_l.font->Release(cstfont_l.font);

	if (CSTEventState)
		CSTEvent->Release(CSTEvent);

	cursor->Release(cursor);
	layer->Release(layer);
	dfb->Release(dfb);

	return 0;
}
