
#ifndef _CST_DFB_COMMON_H_
#define _CST_DFB_COMMON_H_

#define DFBCHECK(x...)                                              \
({                                                                  \
	if (x != DFB_OK) {                                          \
		fprintf(stderr, "%s <%d>:\n\t", __FILE__, __LINE__);\
		DirectFBErrorFatal(#x, x);                          \
	}                                                           \
})

/* These are used to convert the positions to corresponding VGA positions. */
#define VWIDTH (640.0)
#define VHEIGHT (480.0)
extern float VRatio_width;
extern float VRatio_height;

#define VPOSX(x) ((int)((x) * VRatio_width))
#define VPOSY(y) ((int)((y) * VRatio_height))

enum {
	CSTFONT_BLACK = 0,
	CSTFONT_RED,
	CSTFONT_GREEN,
	CSTFONT_YELLOW,
	CSTFONT_BLUE,
	CSTFONT_PURPLE,
	CSTFONT_AQUA,
	CSTFONT_WHITE,
};

enum {
	DUET_INSERTCARD = 0x0001,
};

struct WindowHandle;
typedef int (EventProc)(struct WindowHandle *, DFBWindowEvent *);

typedef struct BackgroundHandle {
	IDirectFBSurface *master_sf;
	IDirectFBSurface *primary_sf;
	char *img;
} HBG_t;

typedef struct WindowHandle {
	IDirectFBWindow *win;
	IDirectFBSurface *master_sf;
	IDirectFBSurface *primary_sf;
	IDirectFBSurface *secondary_sf;  
	DFBWindowID id;
	EventProc *proc;
	char *img_primary;
	char *img_secondary;
	int data;
} HWND_t;

typedef struct _ICSTFont {
	IDirectFBFont *font;
	int FontHeight;
	int NextLineOffset;
} ICSTFont;

extern ICSTFont cstfont_s;
extern ICSTFont cstfont_m;
extern ICSTFont cstfont_l;

extern IDirectFBEventBuffer *CSTEvent;

extern int CSTDFBCommon_DrawString(IDirectFBSurface *surface,
				   ICSTFont *cst_font,
				   int color, char *str, int x, int y);
extern int CSTDFBCommon_WarpCursor(int x, int y);
extern int CSTDFBCommon_WindowActivate(HWND_t *hWnd);
extern int CSTDFBCommon_WindowInactivate(HWND_t *hWnd);
extern int CSTDFBCommon_UpdateSurface(HWND_t *hWnd,
				      IDirectFBSurface *surface);
extern int CSTDFBCommon_LoadImage(IDirectFBSurface *surface,
				  char *image_file);
extern void CSTDFBCommon_ReleaseWindow(HWND_t *hWnd);
extern int CSTDFBCommon_CreateWindow(HWND_t *hWnd,
				     int x, int y, int width, int height,
				     EventProc *proc);
extern void CSTDFBCommon_EventClear(void);
extern void CSTDFBCommon_EventPost(DFBEvent *event);
extern void CSTDFBCommon_EventInitialize(void);
extern int CSTDFBCommon_EventRegistration(HWND_t *hWnd);
extern int CSTDFBCommon_PopupMsg(HBG_t *hBack, int width, char *msg, int sec);
extern int CSTDFBCommon_UpdateBackground(HBG_t *hBack);
extern void CSTDFBCommon_RefreshBackground(HBG_t *hBack);
extern void CSTDFBCommon_ReleaseBackground(HBG_t *hBack);
extern int CSTDFBCommon_CreateBackground(HBG_t *hBack, char *image_file);
extern int CSTDirectFBSetup(int argc, char *argv[]);
extern int CSTDirectFBRelease(void);




#endif /* _CST_DFB_COMMON_H_ */

