#include <directfb.h>

#include <stdio.h>

#include "cst.h"
#include "cst_dfb_common.h"
#include "cst_survey.h"
#include "cst_string.h"
#include "cst_card.h"

static HBG_t hBackground = {
	.img	= RESOURCE_DIR"background.jpg",
};

static HWND_t hSkipButton = {
	.img_primary	= RESOURCE_DIR"icon-skip-up.jpg",
	.img_secondary	= RESOURCE_DIR"icon-skip-down.jpg",
};

static HWND_t hFinishButton = {
	.img_primary	= RESOURCE_DIR"icon-finish-down.jpg",
};

static HWND_t hYesButton = {
	.img_primary	= RESOURCE_DIR"icon-yes-up.jpg",
	.img_secondary	= RESOURCE_DIR"icon-yes-down.jpg",
};

static HWND_t hNoButton = {
	.img_primary	= RESOURCE_DIR"icon-no-up.jpg",
	.img_secondary	= RESOURCE_DIR"icon-no-down.jpg",
};

static HWND_t hCard;

static int
CSTStartWindow_ButtonProc(HWND_t *hWnd, DFBWindowEvent *evt)
{
	static int clicked = 0;

	switch (evt->type) {
	case DWET_GOTFOCUS:
		CSTDFBCommon_UpdateSurface(hWnd, hWnd->secondary_sf);
		CSTDFBCommon_UpdateBackground(&hBackground);
		break;
	case DWET_LOSTFOCUS:
		CSTDFBCommon_UpdateSurface(hWnd, hWnd->primary_sf);
		CSTDFBCommon_UpdateBackground(&hBackground);
		if(clicked)
			clicked = 0;
		break;
	case DWET_BUTTONDOWN:
		if (evt->button == DIBI_LEFT)
			clicked = 1;
		break;
	case DWET_BUTTONUP:
		if (evt->button == DIBI_LEFT) {
			if (clicked) {
				CSTDFBCommon_UpdateSurface(hWnd,
							   hWnd->primary_sf);
				return 1;
			}
		}
		break;
	default:
		break;
	}
	return 0;
}

static int
CSTStartWindow_FinishProc(HWND_t *hWnd, DFBWindowEvent *evt)
{
	static int clicked = 0;

	switch (evt->type) {
	case DWET_GOTFOCUS:
		hWnd->win->SetOpacity(hWnd->win, 0xff);
		CSTDFBCommon_UpdateBackground(&hBackground);
		clicked = 0;
		break;
	case DWET_LOSTFOCUS:
		hWnd->win->SetOpacity(hWnd->win, 0x01);
		CSTDFBCommon_UpdateBackground(&hBackground);
		if(clicked)
			clicked = 0;
		break;
	case DWET_BUTTONDOWN:
		if (evt->button == DIBI_LEFT)
			clicked = 1;
		break;
	case DWET_BUTTONUP:
		if (evt->button == DIBI_LEFT) {
			if (clicked) {
				hWnd->win->SetOpacity(hWnd->win, 0x01);
				return 1;
			}
		}
		break;
	default:
		break;
	}
	return 0;
}

static void
CSTStartWindow_DrawSystemInfo(HBG_t *hBack)
{
	char msg[256];
	int posx = VPOSX(5), posy = VPOSY(440);

	sprintf(msg, " IP ADDR: %s", CSTGetIPaddrString());
	CSTDFBCommon_DrawString(hBackground.master_sf, &cstfont_s, CSTFONT_RED,
				msg, posx, posy);

	posy += cstfont_s.NextLineOffset + VPOSY(1);
	sprintf(msg, "MAC ADDR: %s", CSTGetMACaddrString());
	CSTDFBCommon_DrawString(hBackground.master_sf, &cstfont_s, CSTFONT_RED,
				msg, posx, posy);
}

void
CSTStartWindow_ReleaseResource(void)
{
	CSTDFBCommon_ReleaseBackground(&hBackground);
	CSTDFBCommon_ReleaseWindow(&hSkipButton);
	CSTDFBCommon_ReleaseWindow(&hFinishButton);
	CSTDFBCommon_ReleaseWindow(&hYesButton);
	CSTDFBCommon_ReleaseWindow(&hNoButton);
	CSTDFBCommon_ReleaseWindow(&hCard);
}

int
CSTStartWindow_PrepareResource(void)
{
	CSTFUNC();

	CSTDFBCommon_CreateBackground(&hBackground, hBackground.img);

	CSTDFBCommon_CreateWindow(&hSkipButton,
				  VPOSX(520), VPOSY(430),
				  VPOSX(120), VPOSY(50),
				  CSTStartWindow_ButtonProc);
	CSTDFBCommon_LoadImage(hSkipButton.primary_sf,
			       hSkipButton.img_primary);
	CSTDFBCommon_LoadImage(hSkipButton.secondary_sf,
			       hSkipButton.img_secondary);
	CSTDFBCommon_UpdateSurface(&hSkipButton, hSkipButton.primary_sf);

	CSTDFBCommon_CreateWindow(&hYesButton,
				  VPOSX(520), VPOSY(430),
				  VPOSX(120), VPOSY(50),
				  CSTStartWindow_ButtonProc);
	CSTDFBCommon_LoadImage(hYesButton.primary_sf,
			       hYesButton.img_primary);
	CSTDFBCommon_LoadImage(hYesButton.secondary_sf,
			       hYesButton.img_secondary);
	CSTDFBCommon_UpdateSurface(&hYesButton, hYesButton.primary_sf);

	CSTDFBCommon_CreateWindow(&hNoButton,
				  VPOSX(0), VPOSY(430),
				  VPOSX(120), VPOSY(50),
				  CSTStartWindow_ButtonProc);
	CSTDFBCommon_LoadImage(hNoButton.primary_sf,
			       hNoButton.img_primary);
	CSTDFBCommon_LoadImage(hNoButton.secondary_sf,
			       hNoButton.img_secondary);
	CSTDFBCommon_UpdateSurface(&hNoButton, hNoButton.primary_sf);

	CSTDFBCommon_CreateWindow(&hCard,
				  VPOSX(90), VPOSY(110),
				  VPOSX(460), VPOSY(300),
				  NULL);

	CSTDFBCommon_CreateWindow(&hFinishButton,
				  VPOSX(0), VPOSY(0),
				  VPOSX(120), VPOSY(50),
				  CSTStartWindow_FinishProc);
	CSTDFBCommon_LoadImage(hFinishButton.primary_sf,
			       hFinishButton.img_primary);
	CSTDFBCommon_UpdateSurface(&hFinishButton, hFinishButton.primary_sf);

	return 0;
}

int
CSTStartWindow_Create(int sheet_id)
{
	int posx = VPOSX(5);
	int posy = cstfont_s.NextLineOffset + VPOSY(1 + 5);

	CSTFUNC();

	switch (sheet_id) {
	case 1:
		CSTDFBCommon_WindowActivate(&hSkipButton);
		CSTDFBCommon_EventRegistration(&hSkipButton);

		CSTDFBCommon_DrawString(hBackground.master_sf,
					&cstfont_l, CSTFONT_WHITE,
					CST_STR_CARD, posx, posy);

		CSTStartWindow_DrawSystemInfo(&hBackground);
		break;
	case 2:
		CSTDFBCommon_WindowActivate(&hYesButton);
		CSTDFBCommon_WindowActivate(&hNoButton);
		CSTDFBCommon_EventRegistration(&hYesButton);
		CSTDFBCommon_EventRegistration(&hNoButton);

		CSTDFBCommon_WindowActivate(&hCard);

		CSTDFBCommon_DrawString(hBackground.master_sf,
					&cstfont_l, CSTFONT_WHITE,
					CST_STR_CARDOK, posx, posy);
		break;
	default:
		break;
	}

	hFinishButton.win->SetOpacity(hFinishButton.win, 0x01);
	CSTDFBCommon_EventRegistration(&hFinishButton);

	CSTDFBCommon_UpdateBackground(&hBackground);

	return 0;
}

static int
CSTStartWindow_Exit(int retval)
{
	CSTDFBCommon_EventInitialize();

	CSTDFBCommon_RefreshBackground(&hBackground);
	CSTDFBCommon_UpdateBackground(&hBackground);

	CSTDFBCommon_WarpCursor(VPOSX(320), VPOSY(240));

	CSTDFBCommon_WindowInactivate(&hSkipButton);
	CSTDFBCommon_WindowInactivate(&hYesButton);
	CSTDFBCommon_WindowInactivate(&hNoButton);
	CSTDFBCommon_WindowInactivate(&hFinishButton);
	CSTDFBCommon_WindowInactivate(&hCard);

	return retval;
}

int
CSTStartWindow_Proc(int sheet_id, CSTProcResult_t *result)
{
	DFBEvent evt;
	int ret;
	char file[FILENAME_MAX];

	CSTFUNC();

	while(1) {
		DFBCHECK(CSTEvent->WaitForEvent(CSTEvent));

		while (CSTEvent->HasEvent(CSTEvent) == DFB_OK) {
			DFBCHECK(CSTEvent->GetEvent(CSTEvent, &evt));

			if (evt.clazz == DFEC_WINDOW) {
				if (sheet_id == 0x0001 &&
				    evt.window.window_id == hSkipButton.id) {
					ret = hSkipButton.proc(&hSkipButton,
							       &evt.window);
					if (ret == 1) {
					  CSTSurveyWindow_PrepareCard(NULL);
					  result->type = PROC_RESULT_NEXT;
					  result->page = PROC_PAGE_FINISH;
					  return CSTStartWindow_Exit(0);
					}
				}

				if (sheet_id == 0x0002 &&
				    evt.window.window_id == hYesButton.id) {
					ret = hYesButton.proc(&hYesButton,
							      &evt.window);
					if (ret == 1) {
					  sprintf(file, "%s.jpg",
						  CSTGetFileLabel());
					  CSTSurveyWindow_PrepareCard(file);
					  result->type = PROC_RESULT_NEXT;
					  result->page = PROC_PAGE_FINISH;
					  return CSTStartWindow_Exit(0);
					}
				}

				if (sheet_id == 0x0002 &&
				    evt.window.window_id == hNoButton.id) {
					ret = hNoButton.proc(&hNoButton,
							     &evt.window);
					if (ret == 1) {
					  result->type = PROC_RESULT_CANCEL;
					  return CSTStartWindow_Exit(0);
					}
				}

				if (sheet_id == 0x0001 &&
				    evt.window.window_id == hFinishButton.id) {
					ret = hFinishButton.proc(&hFinishButton,
								 &evt.window);
					if (ret == 1) {
					  result->type = PROC_RESULT_EXIT;
					  return CSTStartWindow_Exit(0);
					}
				}
			} else if (evt.clazz == DFEC_USER) {
				if (sheet_id == 0x0001 &&
				    evt.user.type == DUET_INSERTCARD) {
				  CSTCardResult_t *cardres =
				    (CSTCardResult_t *)evt.user.data;
				  CSTINFO("EVENT(CARD INSERTION)\n");
				  CSTSetFileLabel(cardres->label);
				  CSTReleaseLabel(cardres->label);

				  sprintf(file, "%s.jpg", CSTGetFileLabel());
				  CSTDFBCommon_LoadImage(hCard.primary_sf,
							 file);
				  CSTDFBCommon_UpdateSurface(&hCard,
							     hCard.primary_sf);

				  if (cardres->code == -1)
				    CSTDFBCommon_PopupMsg(&hBackground, 340,
							  CST_STR_DISK, 2);
				  result->type = PROC_RESULT_NEXT;
				  result->page = 0x0002;
				  return CSTStartWindow_Exit(0);
				}
			}
		}
	}
}

