#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <directfb.h>

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "cst.h"
#include "cst_dfb_common.h"
#include "cst_survey.h"
#include "cst_string.h"

#define MIN(a, b) ((a) > (b) ? (b) : (a))
#define IS_SET(a, b) (((a) & (b)) == (b) ? 1 : 0)

static HBG_t hBackground = {
	.img	= RESOURCE_DIR"background.jpg",
};

static HWND_t hNextButton = {
	.img_primary	= RESOURCE_DIR"icon-next-up.jpg",
	.img_secondary	= RESOURCE_DIR"icon-next-down.jpg",
};

static HWND_t hPrevButton = {
	.img_primary	= RESOURCE_DIR"icon-prev-up.jpg",
	.img_secondary	= RESOURCE_DIR"icon-prev-down.jpg",
};

static HWND_t hSkipButton = {
	.img_primary	= RESOURCE_DIR"icon-skip-down.jpg",
};

static HWND_t hAnswer[12] = {
	{
		.img_primary	= RESOURCE_DIR"icon-1-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-1-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-2-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-2-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-3-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-3-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-4-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-4-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-5-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-5-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-6-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-6-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-7-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-7-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-8-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-8-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-9-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-9-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-10-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-10-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-11-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-11-down.jpg",
	},
	{
		.img_primary	= RESOURCE_DIR"icon-12-up.jpg",
		.img_secondary	= RESOURCE_DIR"icon-12-down.jpg",
	},
};

static HWND_t hCard = {
	.img_primary	= RESOURCE_DIR"image-no-card.jpg",
};

#define QUESTION_LINE_SIZE (30)

static int
CSTSurveyWindow_SingleSelection(int sheet_id, DFBWindowID selected)
{
	SSheet_t *sheet = CSTSurvey_GetSheetInfo(sheet_id);
	int i;

	for (i=0; i<sheet->nr_answer; i++) {
		if (selected == hAnswer[i].id)
			continue;
		if (hAnswer[i].data) {
			CSTDFBCommon_UpdateSurface(&hAnswer[i],
						   hAnswer[i].primary_sf);
			CSTDFBCommon_UpdateBackground(&hBackground);
			hAnswer[i].data = 0;
		}
	}
	return 0;
}

static int
CSTSurveyWindow_TallyUp(int sheet_id)
{
	SSheet_t *sheet = CSTSurvey_GetSheetInfo(sheet_id);
	int i;

	for (i=0; i<sheet->nr_answer; i++) {
		if (hAnswer[i].data)
			sheet->answer[i].result = 1;
	}
	return 0;
}

static int
CSTSurveyWindow_CheckSurveyData(int sheet_id)
{
	SSheet_t *sheet = CSTSurvey_GetSheetInfo(sheet_id);
	int selected = 0;
	int i;

	if (IS_SET(sheet->question.param.flag, QAFLAG_SINGLE)) {
		for (i=0; i<sheet->nr_answer; i++) {
			if (hAnswer[i].data)
				selected++;
		}
		if (selected != 1) {
			CSTDFBCommon_PopupMsg(&hBackground, 320,
					      CST_STR_SELECT, 2);
			CSTDFBCommon_EventClear();
			return -1;
		}
	}
	return 0;
}

static int
CSTSurveyWindow_AnswerProc(HWND_t *hWnd, DFBWindowEvent *evt)
{
	static int clicked = 0;

	switch (evt->type) {
	case DWET_GOTFOCUS:
		if (!hWnd->data) {
			CSTDFBCommon_UpdateSurface(hWnd, hWnd->secondary_sf);
			CSTDFBCommon_UpdateBackground(&hBackground);
		}
		clicked = 0;
		break;
	case DWET_LOSTFOCUS:
		if (!hWnd->data) {
			CSTDFBCommon_UpdateSurface(hWnd, hWnd->primary_sf);
			CSTDFBCommon_UpdateBackground(&hBackground);
		}
		if(clicked)
			clicked = 0;
		break;
	case DWET_BUTTONDOWN:
		if (evt->button == DIBI_LEFT)
			clicked = 1;
		break;
	case DWET_BUTTONUP:
		if (evt->button == DIBI_LEFT) {
			if (clicked) {
				if (hWnd->data) {
				  CSTDFBCommon_UpdateSurface(hWnd,
							     hWnd->primary_sf);
				  CSTDFBCommon_UpdateBackground(&hBackground);
				  hWnd->data = 0;
				}else {
				  CSTDFBCommon_UpdateSurface(hWnd,
							     hWnd->secondary_sf);
				  CSTDFBCommon_UpdateBackground(&hBackground);
				  hWnd->data = 1;
				}
				return 1;
			}
		}
		break;
	default:
		break;
	}
	return 0;
}

static int
CSTSurveyWindow_SkipProc(HWND_t *hWnd, DFBWindowEvent *evt)
{
	static int clicked = 0;

	switch (evt->type) {
	case DWET_GOTFOCUS:
		hWnd->win->SetOpacity(hWnd->win, 0xff);
		CSTDFBCommon_UpdateBackground(&hBackground);
		clicked = 0;
		break;
	case DWET_LOSTFOCUS:
		hWnd->win->SetOpacity(hWnd->win, 0x01);
		CSTDFBCommon_UpdateBackground(&hBackground);
		if(clicked)
			clicked = 0;
		break;
	case DWET_BUTTONDOWN:
		if (evt->button == DIBI_LEFT)
			clicked = 1;
		break;
	case DWET_BUTTONUP:
		if (evt->button == DIBI_LEFT) {
			if (clicked) {
				hWnd->win->SetOpacity(hWnd->win, 0x01);
				return 1;
			}
		}
		break;
	default:
		break;
	}
	return 0;
}

static int
CSTSurveyWindow_ButtonProc(HWND_t *hWnd, DFBWindowEvent *evt)
{
	static int clicked = 0;
	switch (evt->type) {
	case DWET_GOTFOCUS:
		CSTDFBCommon_UpdateSurface(hWnd, hWnd->secondary_sf);
		CSTDFBCommon_UpdateBackground(&hBackground);
		clicked = 0;
		break;
	case DWET_LOSTFOCUS:
		CSTDFBCommon_UpdateSurface(hWnd, hWnd->primary_sf);
		CSTDFBCommon_UpdateBackground(&hBackground);
		if(clicked)
			clicked = 0;
		break;
	case DWET_BUTTONDOWN:
		if (evt->button == DIBI_LEFT)
			clicked = 1;
		break;
	case DWET_BUTTONUP:
		if (evt->button == DIBI_LEFT) {
			if (clicked) {
				CSTDFBCommon_UpdateSurface(hWnd,
							   hWnd->primary_sf);
				return 1;
			}
		}
		break;
	default:
		break;
	}
	return 0;
}

static void
CSTSurveyWindow_DrawID(HBG_t *hBack)
{
	int posx = VPOSX(5), posy = VPOSY(5);
	char *label = CSTGetFileLabel();
	char id[FILENAME_MAX];

	label = strcasestr(label, "id_");
	sprintf(id, "ID %s", &label[3]);
	CSTDFBCommon_DrawString(hBack->master_sf, &cstfont_s, CSTFONT_WHITE,
				id, posx, posy);
}

static void
CSTSurveyWindow_DrawPageNo(HBG_t *hBack, int sheet_id)
{
	int posx, posy;
	char buf[256];
	int top_id = CSTSurvey_GetTopSheetID();
	SSheet_t *sheet = CSTSurvey_GetSheetInfo(top_id);
	int total = 0;

	while (sheet) {
		total++;
		sheet = sheet->next;
	}

	posx = VPOSX(300);
	posy = VPOSY(480 - 5) - cstfont_m.NextLineOffset;
	sprintf(buf, "[%d / %d]", sheet_id, total);
	CSTDFBCommon_DrawString(hBack->master_sf, &cstfont_m, CSTFONT_BLACK,
				buf, posx, posy);
}

static int
CSTSurveyWindow_DrawQuestion(HBG_t *hBack, int sheet_id)
{  
	SSheet_t *sheet = CSTSurvey_GetSheetInfo(sheet_id);
	char *msg, *ptr;
	int msg_len;
	char line1[256], line2[256];
	char *line = line1;
	int len = 0;
	int posx = VPOSX(5), posy = VPOSY(5);

	memset(line1, 0, 256);
	memset(line2, 0, 256);

	msg = ptr = sheet->question.msg;
	msg_len = strlen(msg);

	while (ptr - msg < msg_len && len < QUESTION_LINE_SIZE * 2) {
		if (*ptr > 0x80){
			if (len <= QUESTION_LINE_SIZE &&
			    (len + 2) > QUESTION_LINE_SIZE) {
				line = line2;
				len = QUESTION_LINE_SIZE;
			}
			*(line++) = *(ptr++);
			*(line++) = *(ptr++);
			*(line++) = *(ptr++);
			len += 2;
		} else {
			if (len <= QUESTION_LINE_SIZE &&
			    (len + 1) > QUESTION_LINE_SIZE) {
			  line = line2;
			}
			*(line++) = *(ptr++);
			len += 1;
		}
	}

	posy += cstfont_s.NextLineOffset + VPOSY(1 + 5);
	CSTDFBCommon_DrawString(hBack->master_sf, &cstfont_l, CSTFONT_WHITE,
				line1, posx, posy);
	posy += cstfont_l.NextLineOffset + VPOSY(1);
	CSTDFBCommon_DrawString(hBack->master_sf, &cstfont_l, CSTFONT_WHITE,
				line2, posx, posy);

	return 0;
}

void
CSTSurveyWindow_PrepareCard(char *image_file)
{
	CSTFUNC();

	if (image_file) {
		CSTDFBCommon_LoadImage(hCard.secondary_sf, image_file);
		CSTDFBCommon_UpdateSurface(&hCard, hCard.secondary_sf);
	} else
		CSTDFBCommon_UpdateSurface(&hCard, hCard.primary_sf);
}

int
CSTSurveyWindow_PrepareResource(void)
{
	int i;

	CSTFUNC();

	CSTDFBCommon_CreateBackground(&hBackground, hBackground.img);

	for (i=0; i<6; i++) {
		CSTDFBCommon_CreateWindow(&hAnswer[i],
					  VPOSX(20), VPOSY(120 + i * 50),
					  VPOSX(40), VPOSY(40),
					  CSTSurveyWindow_AnswerProc);
		CSTDFBCommon_LoadImage(hAnswer[i].primary_sf,
				       hAnswer[i].img_primary);
		CSTDFBCommon_LoadImage(hAnswer[i].secondary_sf,
				       hAnswer[i].img_secondary);
		CSTDFBCommon_UpdateSurface(&hAnswer[i], hAnswer[i].primary_sf);
	}
	for (; i<12; i++) {
		CSTDFBCommon_CreateWindow(&hAnswer[i],
					  VPOSX(340), VPOSY(120 + (i-6) * 50),
					  VPOSX(40), VPOSY(40),
					  CSTSurveyWindow_AnswerProc);
		CSTDFBCommon_LoadImage(hAnswer[i].primary_sf,
				       hAnswer[i].img_primary);
		CSTDFBCommon_LoadImage(hAnswer[i].secondary_sf,
				       hAnswer[i].img_secondary);
		CSTDFBCommon_UpdateSurface(&hAnswer[i], hAnswer[i].primary_sf);
	}

	CSTDFBCommon_CreateWindow(&hNextButton,
				  VPOSX(520), VPOSY(430),
				  VPOSX(120), VPOSY(50),
				  CSTSurveyWindow_ButtonProc);
	CSTDFBCommon_LoadImage(hNextButton.primary_sf,
			       hNextButton.img_primary);
	CSTDFBCommon_LoadImage(hNextButton.secondary_sf,
			       hNextButton.img_secondary);
	CSTDFBCommon_UpdateSurface(&hNextButton, hNextButton.primary_sf);

	CSTDFBCommon_CreateWindow(&hPrevButton,
				  VPOSX(0), VPOSY(430),
				  VPOSX(120), VPOSY(50),
				  CSTSurveyWindow_ButtonProc);
	CSTDFBCommon_LoadImage(hPrevButton.primary_sf,
			       hPrevButton.img_primary);
	CSTDFBCommon_LoadImage(hPrevButton.secondary_sf,
			       hPrevButton.img_secondary);
	CSTDFBCommon_UpdateSurface(&hPrevButton, hPrevButton.primary_sf);

	CSTDFBCommon_CreateWindow(&hCard,
				  VPOSX(480), VPOSY(0),
				  VPOSX(160), VPOSY(105),
				  NULL);
	CSTDFBCommon_LoadImage(hCard.primary_sf, hCard.img_primary);
	CSTDFBCommon_UpdateSurface(&hCard, hCard.primary_sf);

	CSTDFBCommon_CreateWindow(&hSkipButton,
				  VPOSX(520), VPOSY(0),
				  VPOSX(120), VPOSY(50),
				  CSTSurveyWindow_SkipProc);
	CSTDFBCommon_LoadImage(hSkipButton.primary_sf,
			       hSkipButton.img_primary);
	CSTDFBCommon_UpdateSurface(&hSkipButton, hSkipButton.primary_sf);

	return 0;
}

void
CSTSurveyWindow_ReleaseResource(void)
{
	int i;

	CSTDFBCommon_ReleaseBackground(&hBackground);
	CSTDFBCommon_ReleaseWindow(&hNextButton);
	CSTDFBCommon_ReleaseWindow(&hPrevButton);
	CSTDFBCommon_ReleaseWindow(&hSkipButton);
	for (i=0; i<12; i++)
		CSTDFBCommon_ReleaseWindow(&hAnswer[i]);
	CSTDFBCommon_ReleaseWindow(&hCard);
}

int
CSTSurveyWindow_Create(int sheet_id)
{
	int i;
	SSheet_t *sheet = CSTSurvey_GetSheetInfo(sheet_id);

	CSTFUNC();

	if (!sheet)
		return -1;

	CSTDFBCommon_RefreshBackground(&hBackground);

	for (i=0; i<sheet->nr_answer; i++) {
		if (sheet->answer[i].result) {
			CSTDFBCommon_UpdateSurface(&hAnswer[i],
						   hAnswer[i].secondary_sf);
			hAnswer[i].data = 1;
		} else {
			CSTDFBCommon_UpdateSurface(&hAnswer[i],
						   hAnswer[i].primary_sf);
			hAnswer[i].data = 0;
		}

		CSTDFBCommon_WindowActivate(&hAnswer[i]);
		CSTDFBCommon_EventRegistration(&hAnswer[i]);
	}
	for (i=0; i<MIN(sheet->nr_answer, 6); i++) {
		CSTDFBCommon_DrawString(hBackground.master_sf, &cstfont_m,
					CSTFONT_BLACK, sheet->answer[i].msg,
					VPOSX(70), VPOSY(125 + i * 50));
	}
	for (; i<sheet->nr_answer; i++) {
		CSTDFBCommon_DrawString(hBackground.master_sf, &cstfont_m,
					CSTFONT_BLACK, sheet->answer[i].msg,
					VPOSX(390), VPOSY(125 + (i-6) * 50));
	}

	CSTDFBCommon_WindowActivate(&hNextButton);
	CSTDFBCommon_EventRegistration(&hNextButton);

	CSTDFBCommon_WindowActivate(&hPrevButton);
	CSTDFBCommon_EventRegistration(&hPrevButton);

	hSkipButton.win->SetOpacity(hSkipButton.win, 0x01);
	CSTDFBCommon_EventRegistration(&hSkipButton);

	CSTSurveyWindow_DrawID(&hBackground);
	CSTSurveyWindow_DrawQuestion(&hBackground, sheet_id);
	CSTSurveyWindow_DrawPageNo(&hBackground, sheet_id);

	CSTDFBCommon_WindowActivate(&hCard);

	CSTDFBCommon_UpdateBackground(&hBackground);

	return 0;
}

static int
CSTSurveyWindow_Exit(int retval)
{
	int i;

	CSTFUNC();

	CSTDFBCommon_EventInitialize();

	CSTDFBCommon_RefreshBackground(&hBackground);
	CSTDFBCommon_UpdateBackground(&hBackground);

	CSTDFBCommon_WarpCursor(VPOSX(320), VPOSY(240));
	CSTDFBCommon_WindowInactivate(&hNextButton);
	CSTDFBCommon_WindowInactivate(&hPrevButton);
	CSTDFBCommon_WindowInactivate(&hSkipButton);

	for (i=0; i<12; i++)
		CSTDFBCommon_WindowInactivate(&hAnswer[i]);

	CSTDFBCommon_WindowInactivate(&hCard);

	return retval;
}

int
CSTSurveyWindow_Proc(int sheet_id, CSTProcResult_t *result)
{
	SSheet_t *sheet = CSTSurvey_GetSheetInfo(sheet_id);
	DFBEvent evt;
	int ret;
	int i;

	CSTFUNC();

	if (!sheet) {
		result->type = PROC_RESULT_ERROR;
		return CSTSurveyWindow_Exit(-1);
	}

	while(1) {
		DFBCHECK(CSTEvent->WaitForEvent(CSTEvent));

		while (CSTEvent->HasEvent(CSTEvent) == DFB_OK) {
			DFBCHECK(CSTEvent->GetEvent(CSTEvent, &evt));

			if (evt.clazz == DFEC_WINDOW) {
				if (evt.window.window_id == hNextButton.id) {
				  ret = hNextButton.proc(&hNextButton,
							 &evt.window);
				  if (ret == 1) {
				    ret = CSTSurveyWindow_CheckSurveyData(sheet_id);
				    if (ret < 0)
				      continue;
				    CSTSurveyWindow_TallyUp(sheet_id);

				    result->type = PROC_RESULT_NEXT;
				    if (CSTSurvey_GetSheetInfo(sheet_id + 1))
				      result->page = sheet_id + 1;
				    else
				      result->page = PROC_PAGE_FINISH;
				    return CSTSurveyWindow_Exit(0);
				  }
				}

				else if (evt.window.window_id ==
					 hPrevButton.id) {
				  ret = hPrevButton.proc(&hPrevButton,
							 &evt.window);
				  if (ret == 1) {
				    result->type = PROC_RESULT_CANCEL;
				    return CSTSurveyWindow_Exit(0);
				  }
				}

				else if (evt.window.window_id ==
					 hSkipButton.id) {
				  ret = hSkipButton.proc(&hSkipButton,
							 &evt.window);
				  if (ret == 1) {
				    result->type = PROC_RESULT_ALLCANCEL;
				    return CSTSurveyWindow_Exit(0);
				  }
				}

				else {
				  for (i=0; i<sheet->nr_answer; i++) {
				    if (evt.window.window_id ==
					hAnswer[i].id) {
				      ret = hAnswer[i].proc(&hAnswer[i],
							    &evt.window);
				      if (ret == 1) {
					if (IS_SET(sheet->question.param.flag,
						   QAFLAG_SINGLE))
					  CSTSurveyWindow_SingleSelection(sheet_id,
									  evt.window.window_id);
				      }
				    }
				  }
				}
			}
		}
	}
}
